@@ -1,7 +1,6 @@
 
-
 get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot = 0, jack = 0, Int, int.val = 1, trim = NULL, maxit = 500, cal.epsilon = 1e-04, end.pre, bounds = c(-Inf, Inf), calfun = "raking", qpmeth = "LowRankQP",
-    check.feas = FALSE, use.backup = TRUE, scale.var = "Intercept", cut.mse = 1, time.names = NULL, keep.int = FALSE, printFlag = TRUE, n.cores = 1) {
+    check.feas = FALSE, use.backup = TRUE, scale.var = "Intercept", cut.mse = 1, cut.mse1 = Inf, time.names = NULL, keep.int = FALSE, printFlag = TRUE, n.cores = 1) {
     # Primary function used for calculating weights
 
     # Use LowRankQP if available.  If not, use ipop
@@ -50,7 +49,7 @@ get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot =
             if (printFlag) {
                 message("First model is infeasible: Time = ", round(tmp[3], 2), "\n", sep = "", appendLF = FALSE)
             }
-            dum.tmp <- merge.dums(dum, dum1)
+            dum.tmp <- merge_dums(dum, dum1)
             tmp <- proc.time()
             if (printFlag) {
                 message("Checking feasibility of second model...", "\n", sep = "", appendLF = FALSE)
@@ -79,11 +78,11 @@ get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot =
     newdat <- get.newdat(bigdat, dum = dum, dum1 = dum1, covar.var = covar.var, covar.var1 = covar.var1, end.pre = end.pre, time.names = time.names)
     newdat1 <- newdat[[2]]
     newdat <- newdat[[1]]
-    duma <- merge.dums(dum, dum1)
+    duma <- merge_dums(dum, dum1)
     newdata <- get.newdat(bigdat, dum = duma[[1]], dum1 = duma[[2]], covar.var = covar.var, covar.var1 = covar.var1, end.pre = end.pre, time.names = time.names)
     newdat1a <- newdata[[2]]
     newdata <- newdata[[1]]
-    dumb <- merge.dums(duma[[1]], duma[[2]])
+    dumb <- merge_dums(duma[[1]], duma[[2]])
     covar.var1b <- union(covar.var, covar.var1)
     covar.varb <- NULL
     newdatb <- get.newdat(bigdat, dum = dumb[[1]], dum1 = dumb[[2]], covar.var = covar.varb, covar.var1 = covar.var1b, end.pre = end.pre, time.names = time.names)
@@ -361,6 +360,7 @@ get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot =
                 }
             }
         }
+
     }
 
     if (boot + jack + 1 > for.max) {
@@ -394,7 +394,7 @@ get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot =
         }
     }
 
-    keep.groups <- mse[2 * (use.model - 1) + 1, ] < cut.mse & !is.na(mse[2 * (use.model - 1) + 1, ])
+    keep.groups <- mse[2 * (use.model - 1) + 1, ] < cut.mse1 & !is.na(mse[2 * (use.model - 1) + 1, ])
     keep.groups[!grepl("Perm", colnames(mse))] <- TRUE
 
     if (printFlag & boot > 0) {
@@ -408,28 +408,28 @@ get.w <- function(bigdat, covar.var, covar.var1 = NULL, dum, dum1 = NULL, boot =
 
 
 # Sub-function of get.w()
-merge.dums <- function(dum, dum1) {
+merge_dums <- function(dum, dum1) {
     nam1 <- names(dum)
     nam2 <- names(dum1)
-
+    
     if (length(nam1) > 0) {
         nam <- union(nam1, nam2)
-
+        
         dum.out <- list()
         dum.out[[length(nam)]] <- NA
         dum.new <- list()
         dum.new[[length(nam1)]] <- NA
         names(dum.new) <- nam1
-
+        
         dum[[length(nam) + 1]] <- NA
         dum1[[length(nam) + 1]] <- NA
-
+        
         max.l.dum <- 0
-
+        
         for (i in 1:length(nam)) {
             here1 <- which(nam1 == nam[i])
             here2 <- which(nam2 == nam[i])
-
+            
             if (length(here1) == 0 & length(here2) == 0) {
                 dum.out[[i]] <- NULL
             } else if (length(here1) > 0 & length(here2) == 0) {
@@ -441,25 +441,25 @@ merge.dums <- function(dum, dum1) {
             } else {
                 max.l.dum <- max(max.l.dum, length(dum[[here1]]))
                 dum.new[[here1]] <- sum(dum[[here1]])
-
+                
                 dum1c <- cumsum(dum[[here1]])
                 dum2c <- cumsum(dum1[[here2]])
-
+                
                 dumc <- union(dum1c, dum2c)
                 dumc <- dumc[order(dumc, decreasing = FALSE)]
-
+                
                 if (length(dumc) > 1) {
                   dumc <- c(dumc[1], dumc[2:length(dumc)] - dumc[1:(length(dumc) - 1)])
                 }
-
+                
                 dum.out[[i]] <- dumc
             }
         }
-
+        
         if (max.l.dum <= 1) {
             dum.new <- list()
         }
-
+        
         names(dum.out) <- nam
     } else {
         dum.new <- dum
@@ -473,9 +473,9 @@ merge.dums <- function(dum, dum1) {
 # Sub-function of get.w(); check if a constraint model has a feasible solution
 is.feasible <- function(bigdat, covar.var, dum, Int, int.val = 1, end.pre, eps = 0.001) {
     n <- dim(bigdat)[1]
-
+    
     newdat <- get.newdat(bigdat, dum = dum, covar.var = covar.var, end.pre = end.pre)[[1]]
-
+    
     intdat <- newdat[Int == int.val, ]
     condat <- newdat[Int != int.val, ]
     targets <- colSums(intdat)
@@ -498,17 +498,19 @@ check.feasible2 <- function(A, b, eps = 1e-07, M = 10000, meth = "LowRankQP") {
     a <- c(rep(0, NCOL(A)), rep(1, 2 * NROW(A)))
     Vmat <- matrix(0, NCOL(A3), 1)
     uvec <- rep(M, NCOL(A3))
-
+    
     if (meth == "LowRankQP") {
-        # requireNamespace('LowRankQP', quietly = TRUE)
-        sup.out <- utils::capture.output(all.root <- LowRankQP::LowRankQP(Vmat = Vmat, dvec = a, Amat = A3, bvec = b, uvec = uvec, method = "SMW", verbose = FALSE, niter = maxit))
+        #requireNamespace("LowRankQP", quietly = TRUE)
+        sup.out <- utils::capture.output(all.root <- LowRankQP::LowRankQP(Vmat = Vmat, dvec = a, 
+            Amat = A3, bvec = b, uvec = uvec, method = "SMW", verbose = FALSE, niter = maxit))
         sol <- all.root$alpha
     } else if (meth == "ipop") {
         requireNamespace("kernlab", quietly = TRUE)
 
-        sup.out <- utils::capture.output(all.root <- kernlab::ipop(H = Vmat, c = a, A = A3, b = b, u = uvec, l = rep(0, length(a)), r = 0, verb = 0, maxiter = maxit))
+        sup.out <- utils::capture.output(all.root <- kernlab::ipop(H = Vmat, c = a, A = A3, 
+            b = b, u = uvec, l = rep(0,length(a)), r = 0, verb = 0, maxiter = maxit))
         sol <- kernlab::primal(all.root)
-    }
+   }
     # w <- sol[1:NCOL(A)]
     sol <- sol[-(1:NCOL(A))]
     sol <- sqrt(mean(sol * sol))
@@ -524,17 +526,20 @@ check.feasible2 <- function(A, b, eps = 1e-07, M = 10000, meth = "LowRankQP") {
 }
 
 
-# Sub-function of is.feasible(); reshape data inaccordance with a stated model structure Produces two data frames: newdat (data used for exact constraints) and newdat1 (data used for proximate constraints)
-get.newdat <- function(bigdat, dum = NULL, dum1 = NULL, covar.var = NULL, covar.var1 = NULL, end.pre, time.names = NULL) {
+# Sub-function of is.feasible(); reshape data inaccordance with a stated model structure
+# Produces two data frames: newdat (data used for exact constraints) and newdat1 (data
+# used for proximate constraints)
+get.newdat <- function(bigdat, dum = NULL, dum1 = NULL, covar.var = NULL, covar.var1 = NULL, 
+    end.pre, time.names = NULL) {
     n <- dim(bigdat)[1]
-
+    
     if (length(time.names) == 0) {
         time.names <- as.character(1:dim(bigdat)[3])
     }
-
+    
     result.var <- names(dum)
     newdat <- list()
-
+    
     if (length(result.var) > 0) {
         for (j in 1:length(result.var)) {
             dum.tmp <- dum[[j]]
@@ -556,39 +561,41 @@ get.newdat <- function(bigdat, dum = NULL, dum1 = NULL, covar.var = NULL, covar.
             for (i in 1:length(lows)) {
                 newdat[[j]][, i] <- rowSums(as.matrix(bigdat[, result.var[j], lows[i]:highs[i]]))
                 if (lows[i] == highs[i]) {
-                  colnames(newdat[[j]])[i] <- paste(result.var[j], ".", time.names[lows[i]], sep = "")
+                  colnames(newdat[[j]])[i] <- paste(result.var[j], ".", time.names[lows[i]], 
+                    sep = "")
                 } else {
-                  colnames(newdat[[j]])[i] <- paste(result.var[j], ".", time.names[lows[i]], ".", time.names[highs[i]], sep = "")
+                  colnames(newdat[[j]])[i] <- paste(result.var[j], ".", time.names[lows[i]], 
+                    ".", time.names[highs[i]], sep = "")
                 }
             }
         }
     }
-
+    
     covar.dat <- NULL
     if (length(covar.var) > 0) {
         covar.dat <- bigdat[, covar.var, 1]
         covar.dat <- as.matrix(covar.dat)
         colnames(covar.dat) <- covar.var
     }
-
+    
     if (length(newdat) == 0) {
         newdat <- NULL
     } else {
         newdat <- data.frame(newdat, check.names = FALSE)
     }
-
+    
     if (length(covar.dat) > 0 & length(newdat) > 0) {
         newdat <- data.frame(covar.dat, newdat, check.names = FALSE)
     } else if (length(covar.dat) > 0 & length(newdat) == 0) {
         newdat <- data.frame(covar.dat, check.names = FALSE)
     }
-
+    
     if (length(newdat) > 0) {
         newdat <- data.frame(Intercept = rep(1, n), newdat, check.names = FALSE)
     } else {
         newdat <- data.frame(Intercept = rep(1, n))
     }
-
+    
     newdat1 <- list()
     if (length(dum1) > 0) {
         result.var1 <- names(dum1)
@@ -612,75 +619,80 @@ get.newdat <- function(bigdat, dum = NULL, dum1 = NULL, covar.var = NULL, covar.
             for (i in 1:length(lows)) {
                 newdat1[[j]][, i] <- rowSums(as.matrix(bigdat[, result.var1[j], lows[i]:highs[i]]))
                 if (lows[i] == highs[i]) {
-                  colnames(newdat1[[j]])[i] <- paste(result.var1[j], ".", time.names[lows[i]], sep = "")
+                  colnames(newdat1[[j]])[i] <- paste(result.var1[j], ".", time.names[lows[i]], 
+                    sep = "")
                 } else {
-                  colnames(newdat1[[j]])[i] <- paste(result.var1[j], ".", time.names[lows[i]], ".", time.names[highs[i]], sep = "")
+                  colnames(newdat1[[j]])[i] <- paste(result.var1[j], ".", time.names[lows[i]], 
+                    ".", time.names[highs[i]], sep = "")
                 }
             }
         }
     }
-
+    
     covar.dat1 <- NULL
     if (length(covar.var1) > 0) {
         covar.dat1 <- bigdat[, covar.var1, 1]
         covar.dat1 <- as.matrix(covar.dat1)
         colnames(covar.dat1) <- covar.var1
     }
-
+    
     if (length(newdat1) == 0) {
         newdat1 <- NULL
     } else {
         newdat1 <- data.frame(newdat1, check.names = FALSE)
     }
-
+    
     if (length(covar.dat1) > 0 & length(newdat1) > 0) {
         newdat1 <- data.frame(covar.dat1, newdat1, check.names = FALSE)
     } else if (length(covar.dat1) > 0 & length(newdat1) == 0) {
         newdat1 <- data.frame(covar.dat1, check.names = FALSE)
     }
-
+    
     if (length(newdat1) > 0) {
         newdat1 <- data.frame(newdat1, check.names = FALSE)
     } else {
         newdat1 <- NULL
     }
-
+    
     return(list(newdat = newdat, newdat1 = newdat1))
-
+    
 }
 
 
 
 # Secondary function used for calculating weights
-get.w.sub <- function(newdat = NULL, newdat1 = NULL, bigdat = NULL, dum = NULL, dum1 = NULL, covar.var = NULL, covar.var1 = NULL, end.pre, samp, use, n = NROW(newdat), maxit = 500, calfun = "raking", bounds = c(-Inf,
-    Inf), epsilon = 1e-04, trim = NULL, qpmeth = "LowRankQP", scale.var = "Intercept", printFlag = TRUE) {
+get.w.sub <- function(newdat = NULL, newdat1 = NULL, bigdat = NULL, dum = NULL, dum1 = NULL, 
+    covar.var = NULL, covar.var1 = NULL, end.pre, samp, use, n = NROW(newdat), maxit = 500, 
+    calfun = "raking", bounds = c(-Inf, Inf), epsilon = 1e-04, trim = NULL, qpmeth = "LowRankQP", 
+    scale.var = "Intercept", printFlag = TRUE) {
     tmp <- proc.time()
-
+    
     if (length(newdat) == 0) {
-        newdat <- get.newdat(bigdat = bigdat, dum = dum, dum1 = dum1, covar.var = covar.var, covar.var1 = covar.var1, end.pre = end.pre)
+        newdat <- get.newdat(bigdat = bigdat, dum = dum, dum1 = dum1, covar.var = covar.var, 
+            covar.var1 = covar.var1, end.pre = end.pre)
         newdat1 <- newdat[[2]]
         newdat <- newdat[[1]]
     }
-
+    
     mult <- sum(samp)/sum(use & samp)
     # mult <- 1
-
+    
     if (length(newdat1) > 0) {
         newdat1 <- as.matrix(newdat1)
     }
-
+    
     intdat <- newdat[samp & use, , drop = FALSE]
     scale.by <- sum(intdat[, scale.var])
     condat <- newdat[!samp & use, , drop = FALSE]
-
+    
     alldat <- newdat[, , drop = FALSE]
     scale.by <- scale.by/sum(alldat[, scale.var])
-
+    
     targets <- colSums(intdat)
     targets <- mult * targets
     # targets <- colSums(newdat[samp, , drop = FALSE])
     init <- rep(mult * NROW(intdat)/NROW(condat), NROW(condat))
-
+    
     if (length(newdat1) > 0) {
         intdat1 <- newdat1[samp & use, , drop = FALSE]
         condat1 <- newdat1[!samp & use, , drop = FALSE]
@@ -689,25 +701,27 @@ get.w.sub <- function(newdat = NULL, newdat1 = NULL, bigdat = NULL, dum = NULL,
         targets1 <- mult * targets1
         # targets1 <- colSums(newdat1[samp, , drop = FALSE])
     }
-
+    
     usevars <- colnames(condat)
-
+    
     if (NCOL(as.matrix(condat)) <= NROW(as.matrix(condat))) {
         rem <- find.sing(crossprod(as.matrix(condat)))
     } else {
         rem <- NULL
     }
     keep <- !is.element(1:NCOL(condat), rem)
-
+    
     form <- paste("~", paste(usevars[keep], collapse = "+", sep = ""), "-1", sep = "")
     form <- stats::formula(form)
-
+    
     ws.init <- ws <- init
-
+    
     if (NCOL(newdat) > 1) {
         tryCatch({
             caldesign <- survey::svydesign(ids = ~0, data = condat[, keep], weights = init)
-            cali2 <- survey::calibrate(design = caldesign, maxit = maxit, formula = form, population = targets[keep], data = condat[, keep], calfun = calfun, bounds = bounds, force = TRUE, epsilon = epsilon)
+            cali2 <- survey::calibrate(design = caldesign, maxit = maxit, formula = form, 
+                population = targets[keep], data = condat[, keep], calfun = calfun, bounds = bounds, 
+                force = TRUE, epsilon = epsilon)
             ws.init <- ws <- stats::weights(cali2)
         }, error = function(e, printFlag = printFlag) {
             if (printFlag) {
@@ -715,26 +729,28 @@ get.w.sub <- function(newdat = NULL, newdat1 = NULL, bigdat = NULL, dum = NULL,
             }
         })
     }
-
+    
     if (length(newdat1) > 0) {
         tmp1 <- proc.time() - tmp
         tmp2 <- proc.time()
-
+        
         condat2 <- condat1
         targets2 <- targets1
-
-        ws <- my.qp(b.init = ws.init, X = t(condat2), a = targets2, Y = t(condat[, keep, drop = FALSE]), c = targets[keep], qpmeth = qpmeth, printFlag = printFlag)
+        
+        ws <- my.qp(b.init = ws.init, X = t(condat2), a = targets2, Y = t(condat[, keep, 
+            drop = FALSE]), c = targets[keep], qpmeth = qpmeth, printFlag = printFlag)
         tmp2 <- proc.time() - tmp2
     }
-
+    
     if (length(trim) > 0) {
         if (length(trim) == 1) {
             trim = c(trim, 1 - trim)
         }
-        cali2 <- survey::trimWeights(cali2, upper = stats::quantile(ws, max(trim)), lower = stats::quantile(ws, min(trim)))
+        cali2 <- survey::trimWeights(cali2, upper = stats::quantile(ws, max(trim)), lower = stats::quantile(ws, 
+            min(trim)))
         ws <- stats::weights(cali2)
     }
-
+    
     mse1 <- mean((colSums(ws * condat[, , drop = FALSE]) - targets)^2)
     wghts1 <- rep(NA, n)
     wghts1[!samp & use] <- ws
@@ -742,24 +758,27 @@ get.w.sub <- function(newdat = NULL, newdat1 = NULL, bigdat = NULL, dum = NULL,
     wghts.init1 <- rep(NA, n)
     wghts.init1[!samp & use] <- ws.init
     wghts.init1[samp & use] <- mult
-
+    
     out <- list(wghts = wghts1, wghts.init = wghts.init1, mse = mse1, scale.by = scale.by)
-
+    
     return(out)
 }
 
 
 
-get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boots, boots, newdat, newdat1, newdata, newdat1a, newdatb, newdat1b, end.pre, maxit, calfun, bounds, cal.epsilon, trim, qpmeth, scale.var,
-    printFlag, cut.mse, use.backup, jack, back.state, back.state1, back.state2, back.state3, tmp.jack, boot, boot.lower, boot.upper, jack.lower, jack.upper, rownams, rownams1) {
-
+get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boots, boots, 
+    newdat, newdat1, newdata, newdat1a, newdatb, newdat1b, end.pre, maxit, calfun, bounds, 
+    cal.epsilon, trim, qpmeth, scale.var, printFlag, cut.mse, use.backup, jack, back.state, 
+    back.state1, back.state2, back.state3, tmp.jack, boot, boot.lower, boot.upper, jack.lower, 
+    jack.upper, rownams, rownams1) {
+    
     tmp.boot <- tmp.jack
     i <- X
     wghts.out <- Inter.out <- rep(NA, n)
     names(wghts.out) <- names(Inter.out) <- rownams
     mse.out <- rep(NA, 6)
     names(mse.out) <- rownams1
-
+    
     use.model.i <- use.model
     tmp <- proc.time()
     if (i == 1) {
@@ -785,9 +804,10 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
     Inter.out[!samp & use] <- FALSE
     if (use.model.i == 1) {
         mod.out <- "First"
-
-        ws <- get.w.sub(newdat = newdat, newdat1 = newdat1, end.pre = end.pre, samp = samp, use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth,
-            scale.var = scale.var, printFlag = printFlag)
+        ws <- get.w.sub(newdat = newdat, newdat1 = newdat1, end.pre = end.pre, samp = samp, 
+            use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, 
+            epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth, scale.var = scale.var, 
+            printFlag = printFlag)
         if (ws$mse > cut.mse | is.na(ws$mse)) {
             if (use.backup) {
                 cat.back <- ".  Using second model."
@@ -796,9 +816,11 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
                 cat.back <- cat.back1 <- ".  Consider setting use.backup = TRUE."
             }
             if (i > 1) {
-                back.state1 <- paste("First model was infeasible for ", rep.meth, " group ", g, cat.back1, "\n", sep = "")
+                back.state1 <- paste("First model was infeasible for ", rep.meth, " group ", 
+                  g, cat.back1, "\n", sep = "")
             } else {
-                back.state1 <- paste("First model is infeasible for primary weights", cat.back, "\n", sep = "")
+                back.state1 <- paste("First model is infeasible for primary weights", cat.back, 
+                  "\n", sep = "")
             }
             if (use.backup) {
                 use.model.i <- 2
@@ -807,25 +829,31 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
     }
     if (use.model.i == 2 & use.backup) {
         mod.out <- "Second"
-
-        ws <- get.w.sub(newdat = newdata, newdat1 = newdat1a, end.pre = end.pre, samp = samp, use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth,
-            scale.var = scale.var, printFlag = printFlag)
+        ws <- get.w.sub(newdat = newdata, newdat1 = newdat1a, end.pre = end.pre, samp = samp, 
+            use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, 
+            epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth, scale.var = scale.var, 
+            printFlag = printFlag)
         if (ws$mse > cut.mse | is.na(ws$mse)) {
             if (i > 1) {
-                back.state2 <- paste("Second model was infeasible for ", rep.meth, " group ", g, ".  Used third model.", "\n", sep = "")
-                back.state3 <- paste("First and second model were infeasible for ", rep.meth, " group ", g, ".  Used third model.", "\n", sep = "")
+                back.state2 <- paste("Second model was infeasible for ", rep.meth, " group ", 
+                  g, ".  Used third model.", "\n", sep = "")
+                back.state3 <- paste("First and second model were infeasible for ", rep.meth, 
+                  " group ", g, ".  Used third model.", "\n", sep = "")
             } else {
-                back.state2 <- paste("Second model is infeasible for primary weights.  Will use third model.", "\n", sep = "")
-                back.state3 <- paste("First and second model are infeasible for primary weights.  Will use third model.", "\n", sep = "")
+                back.state2 <- paste("Second model is infeasible for primary weights.  Will use third model.", 
+                  "\n", sep = "")
+                back.state3 <- paste("First and second model are infeasible for primary weights.  Will use third model.", 
+                  "\n", sep = "")
             }
             use.model.i <- 3
         }
     }
     if (use.model.i == 3 & use.backup) {
         mod.out <- "Third"
-
-        ws <- get.w.sub(newdat = newdatb, newdat1 = newdat1b, end.pre = end.pre, samp = samp, use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth,
-            scale.var = scale.var, printFlag = printFlag)
+        ws <- get.w.sub(newdat = newdatb, newdat1 = newdat1b, end.pre = end.pre, samp = samp, 
+            use = use, n = NROW(newdat), maxit = maxit, calfun = calfun, bounds = bounds, 
+            epsilon = cal.epsilon, trim = trim, qpmeth = qpmeth, scale.var = scale.var, 
+            printFlag = printFlag)
     }
     if (back.state1 != "" & back.state2 != "") {
         back.state.final <- back.state3
@@ -850,14 +878,15 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
     wghts.out[] <- ws$wghts
     tmp <- proc.time() - tmp
     if (i == 1) {
-        if (back.state != "")
+        if (back.state != "") 
             if (printFlag) {
                 message(back.state, appendLF = FALSE)
             }
         back.state <- ""
         use.model <- use.model.i
         if (printFlag) {
-            message("Created main weights for synthetic control: Time = ", round(tmp[3], 2), "\n\n", sep = "", appendLF = FALSE)
+            message("Created main weights for synthetic control: Time = ", round(tmp[3], 
+                2), "\n\n", sep = "", appendLF = FALSE)
         }
         if (printFlag) {
             message("Matching summary for main weights:\n", appendLF = FALSE)
@@ -871,7 +900,8 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
             }
             printstuff <- mses$printstuff
             if (printFlag) {
-                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), appendLF = FALSE)
+                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), 
+                  appendLF = FALSE)
             }
             if (printFlag) {
                 message("\n", appendLF = FALSE)
@@ -885,7 +915,8 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
             }
             printstuff <- msesa$printstuff
             if (printFlag) {
-                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), appendLF = FALSE)
+                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), 
+                  appendLF = FALSE)
             }
             if (printFlag) {
                 message("\n", appendLF = FALSE)
@@ -899,7 +930,8 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
             }
             printstuff <- msesb$printstuff
             if (printFlag) {
-                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), appendLF = FALSE)
+                message(paste0(utils::capture.output(round(printstuff, 4)), collapse = "\n"), 
+                  appendLF = FALSE)
             }
             if (printFlag) {
                 message("\n", appendLF = FALSE)
@@ -941,14 +973,15 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
         if (i == jack.upper) {
             if (i == jack.lower) {
             }
-            if (back.state != "")
+            if (back.state != "") 
                 if (printFlag) {
                   message(back.state, appendLF = FALSE)
                 }
             back.state <- ""
             tmp.jack <- proc.time() - tmp.jack
             if (printFlag) {
-                message("Completed weights for all jackknife replication groups: Time = ", round(tmp.jack[3], 2), "\n\n", sep = "", appendLF = FALSE)
+                message("Completed weights for all jackknife replication groups: Time = ", 
+                  round(tmp.jack[3], 2), "\n\n", sep = "", appendLF = FALSE)
             }
             if (boot > 0) {
                 if (printFlag) {
@@ -960,7 +993,8 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
     } else if (i >= boot.lower & i <= boot.upper) {
         if (i == boot.lower) {
             if (printFlag) {
-                message("Completed weights for permutation group:\n", i - jack - 1, sep = "", appendLF = FALSE)
+                message("Completed weights for permutation group:\n", i - jack - 1, sep = "", 
+                  appendLF = FALSE)
             }
         } else if ((i - jack - 1)%%20 != 1 & i != boot.upper) {
             if (printFlag) {
@@ -987,7 +1021,8 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
             }
             tmp.boot <- proc.time() - tmp.boot
             if (printFlag) {
-                message("Completed weights for all permutation groups: Time = ", round(tmp.boot[3], 2), "\n\n", sep = "", appendLF = FALSE)
+                message("Completed weights for all permutation groups: Time = ", round(tmp.boot[3], 
+                  2), "\n\n", sep = "", appendLF = FALSE)
             }
         }
     }
@@ -998,9 +1033,9 @@ get.w.sub.par <- function(X, use.model, Int, int.val, colnam, rep.G, n, fin.boot
 
 # Determine pre-intervention MSEs and create balance table
 get.mse <- function(newdat, newdat1 = NULL, samp, use, ws, ws.init, scale.by) {
-
+    
     mult <- sum(samp)/sum(use & samp)
-
+    
     intdat <- newdat[samp & use, , drop = FALSE]
     condat <- newdat[!samp & use, , drop = FALSE]
     alldat <- newdat[, , drop = FALSE]
@@ -1008,7 +1043,7 @@ get.mse <- function(newdat, newdat1 = NULL, samp, use, ws, ws.init, scale.by) {
     targets <- mult * targets
     ws <- ws[!samp & use]
     mse <- mean((colSums(ws * condat) - targets)^2)
-
+    
     mse1 <- NA
     if (length(newdat1) != 0) {
         intdat1 <- newdat1[samp & use, , drop = FALSE]
@@ -1019,16 +1054,18 @@ get.mse <- function(newdat, newdat1 = NULL, samp, use, ws, ws.init, scale.by) {
         ws.init <- ws.init[!samp & use]
         mse1 <- mean((colSums(ws * condat1) - targets1)^2)
     }
-
+    
     if (length(newdat1) == 0) {
-        printstuff <- cbind(Targets = targets, Weighted.Control = colSums((ws) * (condat)), All.scaled = (scale.by) * colSums(alldat))
+        printstuff <- cbind(Targets = targets, Weighted.Control = colSums((ws) * (condat)), 
+            All.scaled = (scale.by) * colSums(alldat))
     } else {
-
-        # printstuff <- cbind(Targets = c(targets, targets1), Initial.Weighted.Control = colSums(ws.init * cbind(condat, condat1)), Final.Weighted.Control = colSums(ws * cbind(condat, condat1)), All.scaled =
-        # scale.by * colSums(cbind(alldat, alldat1)))
-        printstuff <- cbind(Targets = c(targets, targets1), Final.Weighted.Control = colSums(ws * cbind(condat, condat1)), All.scaled = scale.by * colSums(cbind(alldat, alldat1)))
+        # printstuff <- cbind(Targets = c(targets, targets1), Initial.Weighted.Control =
+        # colSums(ws.init * cbind(condat, condat1)), Final.Weighted.Control = colSums(ws *
+        # cbind(condat, condat1)), All.scaled = scale.by * colSums(cbind(alldat, alldat1)))
+        printstuff <- cbind(Targets = c(targets, targets1), Final.Weighted.Control = colSums(ws * 
+            cbind(condat, condat1)), All.scaled = scale.by * colSums(cbind(alldat, alldat1)))
     }
-
+    
     return(list(mse = mse, mse1 = mse1, printstuff = printstuff))
 }
 
@@ -1037,16 +1074,16 @@ get.mse <- function(newdat, newdat1 = NULL, samp, use, ws, ws.init, scale.by) {
 my.qp <- function(b.init, X, Y, a, c, M = 10000, qpmeth = "LowRankQP", maxit = 1000, printFlag = TRUE) {
     q <- NCOL(X)
     n <- NROW(Y)
-
+    
     a1 <- iso.nam(a, sep = ".")
     c1 <- iso.nam(c, sep = ".")
-
+    
     a1[a1 == 0] <- 1
     c1[c1 == 0] <- 1
-
-    a1 <- sqrt(a1)
-    c1 <- sqrt(c1)
-
+    
+    a1 <- sqrt(abs(a1))
+    c1 <- sqrt(abs(c1))
+    
     X <- X/a1
     Y <- Y/c1
     a <- a/a1
@@ -1058,35 +1095,81 @@ my.qp <- function(b.init, X, Y, a, c, M = 10000, qpmeth = "LowRankQP", maxit = 1
         rem <- NULL
     }
     leave <- setdiff(1:NROW(Y), rem)
-
-
-    # if (qpmeth == 'nleqslv') { requireNamespace('nleqslv', quietly = TRUE) XtX <- crossprod(X) M <- 1/M jacobian <- function(all) { b <- all[1:q] lambda <- all[(q + 1):(q + n)] lowerright <- matrix(0, n, n)
-    # upperleft <- diag(1/b) + XtX upperright <- t(Y) lowerleft <- Y out <- rbind(cbind(upperleft, upperright), cbind(lowerleft, lowerright)) return((out)) } f1 <- function(all) { b <- all[1:q] lambda <- all[(q +
-    # 1):(q + n)] out1 <- (M * log(b)) + crossprod(X, (X %*% b - a)) - crossprod(Y, lambda) out2 <- Y %*% b - c return(c(out1, out2)) } b.init[b.init < 1e-06] <- 1e-06 lambda.init <- solve(tcrossprod(Y), Y) %*%
-    # ((M * log(b.init)) + crossprod(X, (X %*% b.init - a))) all.init <- c(b.init, lambda.init) all.root <- nleqslv::nleqslv(all.init, fn = f1, jac = NULL, method = c('Broyden'), global = c('gline'), xscalm =
-    # c('fixed'), control = list(maxit = maxit, xtol = 1e-11, ftol = 1e-11, btol = 1e-06, cndtol = 1e-13)) if (printFlag) { message('Number of Jacobian evaluations = ', all.root$njcnt, '.  \n', sep = '', appendLF
-    # = FALSE) } if (printFlag) { message('Number of function evaluations = ', all.root$nfcnt, '. \n', sep = '', appendLF = FALSE) } if (printFlag) { message('Number of iterations = ', all.root$iter, '. \n', sep
-    # = '', appendLF = FALSE) } all.root <- all.root$x b <- all.root[1:q] b[b < 0] <- 0 } else
+    
+    #if (qpmeth == "nleqslv") {
+    #   requireNamespace("nleqslv", quietly = TRUE)
+    #   XtX <- crossprod(X)
+    #   
+    #   M <- 1/M
+    #   jacobian <- function(all) {
+    #       b <- all[1:q]
+    #       lambda <- all[(q + 1):(q + n)]
+    #       lowerright <- matrix(0, n, n)
+    #       upperleft <- diag(1/b) + XtX
+    #       upperright <- t(Y)
+    #       lowerleft <- Y
+    #       out <- rbind(cbind(upperleft, upperright), cbind(lowerleft, lowerright))
+    #       return((out))
+    #   }
+    #   
+    #   f1 <- function(all) {
+    #       b <- all[1:q]
+    #       lambda <- all[(q + 1):(q + n)]
+    #       out1 <- (M * log(b)) + crossprod(X, (X %*% b - a)) - crossprod(Y, lambda)
+    #       out2 <- Y %*% b - c
+    #       return(c(out1, out2))
+    #   }
+    #   b.init[b.init < 1e-06] <- 1e-06
+    #   lambda.init <- solve(tcrossprod(Y), Y) %*% ((M * log(b.init)) + crossprod(X, (X %*% 
+    #       b.init - a)))
+    #   all.init <- c(b.init, lambda.init)
+    #   
+    #   all.root <- nleqslv::nleqslv(all.init, fn = f1, jac = NULL, method = c("Broyden"), 
+    #       global = c("gline"), xscalm = c("fixed"), control = list(maxit = maxit, xtol = 1e-11, 
+    #           ftol = 1e-11, btol = 1e-06, cndtol = 1e-13))
+    #   
+    #   if (printFlag) {
+    #       message("Number of Jacobian evaluations = ", all.root$njcnt, ". \n", sep = "", 
+    #           appendLF = FALSE)
+    #   }
+    #   if (printFlag) {
+    #       message("Number of function evaluations = ", all.root$nfcnt, ". \n", sep = "", 
+    #           appendLF = FALSE)
+    #   }
+    #   if (printFlag) {
+    #       message("Number of iterations = ", all.root$iter, ". \n", sep = "", appendLF = FALSE)
+    #   }
+    #   all.root <- all.root$x
+    #   
+    #   b <- all.root[1:q]
+    #   b[b < 0] <- 0
+    # 
+    #} else 
     if (qpmeth == "LowRankQP") {
-        # requireNamespace('LowRankQP', quietly = TRUE)
+        #requireNamespace("LowRankQP", quietly = TRUE)
 
-        sup.out <- utils::capture.output(all.root <- LowRankQP::LowRankQP(Vmat = t(X), dvec = -crossprod(X, a), Amat = Y[leave, , drop = FALSE], bvec = c[leave], uvec = rep(M, q), method = "SMW", verbose = FALSE, niter = maxit))
+        sup.out <- utils::capture.output(all.root <- LowRankQP::LowRankQP(Vmat = t(X), dvec = -crossprod(X, 
+            a), Amat = Y[leave, , drop = FALSE], bvec = c[leave], uvec = rep(M, q), method = "SMW", 
+            verbose = FALSE, niter = maxit))
         b <- all.root$alpha
     } else if (qpmeth == "ipop") {
         requireNamespace("kernlab", quietly = TRUE)
-
-        sup.out <- utils::capture.output(all.root <- kernlab::ipop(H = t(X), c = -crossprod(X, a), A = Y[leave, , drop = FALSE], b = c[leave], u = rep(M, q), verb = 0, maxiter = maxit))
-        # all.root <- kernlab::ipop(c = -crossprod(X, a), H = crossprod(X), b = c, A = Y, l = rep(0, q), r = rep(0, n), u = rep(M, q))
+        
+        sup.out <- utils::capture.output(all.root <- kernlab::ipop(H = t(X), c = -crossprod(X, 
+            a), A = Y[leave, , drop = FALSE], b = c[leave], u = rep(M, q), 
+            verb = 0, maxiter = maxit))
+        #all.root <- kernlab::ipop(c = -crossprod(X, a), H = crossprod(X), b = c, A = Y, 
+        #    l = rep(0, q), r = rep(0, n), u = rep(M, q))
         b <- kernlab::primal(all.root)
     } else {
         stop("qpmeth not recognized.")
     }
-
+    
     X <- X * a1
     Y <- Y * c1
     a <- a * a1
     c <- c * c1
-
+    
     return(b)
 }
 
@@ -1096,7 +1179,7 @@ iso.nam <- function(a, sep = "-") {
     locs <- regexpr(sep, nams, fixed = TRUE)
     outs <- nams
     outs[locs > 1] <- substr(outs[locs > 1], 1, locs[locs > 1] - 1)
-
+    
     out <- rep(NA, length(outs))
     for (i in names(table(outs))) {
         out[outs == i] <- mean(a[outs == i])
